function [anal] = si_rz(rg,zg,Tb,ro,zo,To,Lr,Lz,sigb,sigo)
% Does a 2-dimensional univaiate statistical interpolation analysis of
% obsevations To at locations (ro,zo) onto a grid (rg,zg) with background
% field Tb. (rg,zg) need to have been made using meshgrid and have constant
% spacing.
% Background error covariances are modelled in cartesian coordinates with
% length scales Lr and Lz, and variance sigb^2. Observation errors
% are assumed uncorrelated with variance sigo^2.
% Follows somewhere between Lorenc (1981) and  Talagrand (1997) section 4.

sigb2 = sigb^2; sigo2 = sigo^2;

[nz,nr] = size(Tb); nrz = nr*nz;
dr = rg(1,2) - rg(1,1); dz = zg(2,1) - zg(1,1);

no = length(ro);

if (size(rg) ~= [nz nr]) | (size(zg) ~= [nz nr])
   disp('Background and grids size mismatch')
   anal = nan;
   return
end

if (size(zo) ~= [no 1]) | (size(To) ~= [no 1])
   disp('Observations and locations size mismatch')
   anal = nan;
   return
end
   
% Clean up observations outside the grid
zgmin = min(zg(:)); zgmax = max(zg(:));
rgmin = min(rg(:)); rgmax = max(rg(:));
use = ( zgmin <= zo & zo <= zgmax & rgmin <= ro & ro <= rgmax );
if (sum(use) < no)
   ro = ro(find(use)); zo = zo(find(use)); To = To(find(use));
   disp(sprintf('WARNING>> Discarding %d out of %d obs off grid ... rejects',no-sum(use),no));
   disp(find(~use));
   no = sum(use);
end

% Clean up missing observations
use = find( ~isnan(To) );
if (length(use) < no)
   ro = ro(use); zo = zo(use); To = To(use);
   disp(sprintf('WARNING>> Discarding %d out of %d obs (missing value)',no-length(use),no));
   no = length(use);
end

Toinc = zeros(no,1);
for io = 1 : no
   ir = max( find( rg(1,:) <= ro(io) ) );
   iz = max( find( zg(:,1) <= zo(io) ) );
   r1 = ( -rg(iz,ir) + ro(io) ) / dr; r2 = 1 - r1;
   z1 = ( -zg(iz,ir) + zo(io) ) / dz; z2 = 1 - z1;
   Toinc(io) = To(io) - r1*(z1*Tb(iz,ir) + z2*Tb(iz+1,ir)) ...
         - r2*(z1*Tb(iz,ir+1) + z2*Tb(iz+1,ir+1));
end;

if assert( any(isnan(Toinc)), 'Toinc has nans' )
   anal = nan; return
end

C = zeros(no);
for io = 1 : no
   C(io,io) = sigb2 + sigo2;
   for jo = io+1 : no
      C(io,jo) = sigb2 * exp( -0.5*dsq( ro(io),zo(io),ro(jo),zo(jo),Lr,Lz ) );
      C(jo,io) = C(io,jo);
   end
end
ZZ = inv(C) * Toinc;

if assert( any(isnan(ZZ)), 'ZZ has nans' )
   anal = nan; return
end

Tainc = zeros(nrz,1);
for kg = 1 : nrz
   HBcol = zeros(no,1);
   for io = 1 : no
      HBcol(io) = sigb2 * exp( -0.5*dsq( ro(io),zo(io),rg(kg),zg(kg),Lr,Lz ) );
   end
   Tainc(kg) = HBcol' * ZZ;  % Analysis incements
end

anal = Tb + reshape( Tainc, [nz,nr] );  % Analysis

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function dd = dsq(r1,z1,r2,z2,Lr,Lz)
% returns the square of the nondimensional distance between two points

dd = ( (r1 - r2)/Lr )^2 + ( (z1 - z2) / Lz )^2;

