function [anal, err] = tcsi(xg,yg,Tb,xo,yo,To,Lrad,Laz,sigb,sigo)
% Does a 2-dimensional univariate statistical interpolation analysis of 
% observations To at locations (xo,yo) onto a grid (xg,yg) with background
% field Tb. (xg,yg) need to have been made using meshgrid and have constant
% spacing.
% Background error covariances are modelled in polar coordinates with 
% length scales Lrad and Laz, and variance sigb^2. Observation errors
% are assumed uncorrelated with variance sigo^2.
% Follows somewhere between Lorenc (1981) and  Talagrand (1997) section 4.

calcB = 0;   % Calculate off-diagonal terms of B

sigb2 = sigb^2; sigo2 = sigo^2;

[ny,nx] = size(xg); nxy = nx*ny;
rg = hypot(xg,yg); thg = atan2(yg,xg); 
dx = xg(1,2) - xg(1,1); dy = yg(2,1) - yg(1,1);

if length(Tb) == 1, Tb = repmat(Tb, [ny,nx]); end;

ro = hypot(xo, yo); tho = atan2(yo,xo); no = length(xo);

H = zeros(no,nxy);
Hx = reshape( 1:nxy , [ny,nx] );
for io = 1 : no
   if 1 % Bi-linear interpolation
      ix = max( find( xg(1,:) <= xo(io) ) );
      iy = max( find( yg(:,1) <= yo(io) ) );
      x1 = ( -xg(iy,ix) + xo(io) ) / dx; x2 = 1 - x1;
      y1 = ( -yg(iy,ix) + yo(io) ) / dy; y2 = 1 - y1;
      H( io, Hx(iy  ,ix  ) ) = x2*y2;
      H( io, Hx(iy+1,ix  ) ) = x2*y1;
      H( io, Hx(iy  ,ix+1) ) = x1*y2;
      H( io, Hx(iy+1,ix+1) ) = x1*y1;
   else  % Lagrange interpolation
      for iy = 1 : ny
         ymask = [ 1 : iy-1 , iy+1 : ny ]';
         d = prod( (yo(io) - yg(ymask,1)) ./ (yg(iy,1) - yg(ymask,1)) );
         for ix = 1 : nx
            xmask = [ 1 : ix-1 , ix+1 : nx ];
            c = prod( (xo(io) - xg(1,xmask)) ./ (xg(1,ix) - xg(1,xmask)) );
            H( io, Hx(iy,ix) ) = c*d;
         end
      end
   end;
end;

C = zeros(no);
for io = 1 : no
   C(io,io) = sigb2 + sigo2;
   for jo = io+1 : no
      C(io,jo) = sigb2 * exp( -0.5*dsq( ro(io),tho(io),ro(jo),tho(jo),Lrad,Laz ) );
      C(jo,io) = C(io,jo);
   end
end
Ci = inv(C);

% Background error matrix (only used for plotting)
B = sigb2*eye(nxy);    
if calcB
   for ig = 1 : nxy
      for jg = ig+1 : nxy
         B(ig,jg) = sigb2 * exp( -0.5*dsq( rg(ig),thg(ig),rg(jg),thg(jg),Lrad,Laz ) );
         B(jg,ig) = B(ig,jg);
      end
   end
end

HB = zeros(no,nxy);
for io = 1 : no
   for kg = 1 : nxy
      HB(io,kg) = sigb2 * exp( -0.5*dsq( ro(io),tho(io),rg(kg),thg(kg),Lrad,Laz ) );
   end
end

Tainc = HB' * Ci * ( To - H*(Tb(:)) );  % Analysis increments
anal = Tb + reshape( Tainc, [ny,nx] );  % Analysis

Pa = B - HB' * Ci * HB;                 % Analysis error covariance matrix
err = reshape( diag(Pa), [ny,nx] );     % Analysis error variances

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function dd = dsq(r1,th1,r2,th2,Lrad,Laz)
% Returns the square of the nondimensional polar distance between two points

dth = abs( th1 - th2 );
if (abs(dth) > pi), 
   dth = 2*pi - dth; 
end;
% dd = ( (r1 - r2)/Lrad )^2 + r1 * r2 * ( dth / Laz )^2; 

dd = ( (r1 - r2)/Lrad )^2 + ( dth / Laz )^2;   % Laz in radians

