function [p,dp] = willoughby1_p_noblend(r,a,f);
% function [p,dp] = willoughby1_p_noblend(r,a,f);
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm) and its derivatives
% with respect to the control vector together with the radial integral of the
% gradient wind equation i.e. the corresponding pressure field.
% *** Simplified version omits the blending zone between v1 and v2. ***
% The control vector a = (vm,L,rm,pc)

if (nargin < 3)
   f = cor(15);
end
rho = 1.15;
vm = a(1);
L  = a(2);
rm = a(3);
pc = a(4);
Li = 1/L; 
nr = length(r);

rrm = r/rm;
p  = zeros(nr,1);
dp = zeros(nr,4);

% Inner zone
ix = find( 0 <= r & r < rm );
p(ix) = pc + rho*0.5*vm*(vm + f*rm)*rrm(ix).^2;
dp(ix,1) = rho*(vm + 0.5*f*rm)*rrm(ix).^2;
dp(ix,2) = 0;
dp(ix,3) = -rho*vm*(vm/rm + 0.5*f)*rrm(ix).^2;
dp(ix,4) = 1;

% Outer zone
ix = find( r >= rm );
Ei1  = -real(expint(2*r(ix)*Li)) + real(expint(2*rm*Li));
e1m = exp(rm*Li); e1 = exp((rm - r(ix))*Li);
p(ix) = pc + rho*( 0.5*vm*(vm + f*rm) ...
      + f*( vm*L*(1 - exp((rm - r(ix))*Li)) ) ...
      + (vm*e1m)^2 * Ei1 );
dp(ix,1) = rho*( (vm + 0.5*f*rm) + f*L*(1 - e1) + 2*vm*e1m^2*Ei1 );
dp(ix,2) = rho*vm*( f*( 1 - e1.*( 1 - Li*(rm - r(ix)) ) ) ...
                 + vm*Li*( (1 - e1.^2) - 2*rm*Li*e1m^2*Ei1 ) );
dp(ix,3) = rho*( 0.5*f*vm - f*vm*e1 - vm^2/rm + 2*(vm*e1m)^2*Li*Ei1 );
dp(ix,4) = 1;

% Special points
ix = find(r == -999.0);
rmhat = 25e3; rmscal = 1;
Lhat = 7e4;  Lscal = 1;
p(ix) = rmscal*(rm/rmhat - 1)^4 + Lscal*(L/Lhat - 1)^4;
dp(ix,1) = 0;
dp(ix,2) = 4*Lscal/Lhat*(L/Lhat - 1)^3;
dp(ix,3) = 4*rmscal/rmhat*(rm/rmhat - 1)^3;
dp(ix,4) = 0;


