function [p,dp] = willoughby1_p_numerical(r,a,f);
% function [p,dp] = willoughby1_p_numerical(r,a,f);
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm) and its derivatives
% with respect to the control vector together with the radial integral of the
% gradient wind equation i.e. the corresponding pressure field.
% Uses willoughby1_v to get the wind field and then numerically integrates it.
% The control vector a = (vm,L,rm,pc)

persistent olda vm L Li rm pc Lb
persistent wm dwmdvm dwmdL dwmdrm 
persistent rsm dwsdrm_rsm dwsdrsi_rsm rblend1 rblend2

if (nargin < 3)
   f = cor(15);
end

if isempty(olda) | any(a ~= olda)  % Initialise on first time or changed a
   %display('***Initialising');
   olda = a;
   
   vm = a(1);   % amplitude of first exponential
   L  = a(2);   % length scale of first exponential
   rm = a(3);   % radius of maximum winds
   pc = a(4);   % central pressure
   Lb = 5e3;    % Half-width of blending zone
   Li = 1/L; 
   
   % Find wm, the weight that must apply at rm, and its derivatives.
   wm = 1 / ( 1 + rm*Li );
   dwmdvm =  0;
   dwmdL  =  wm^2 * rm * Li^2;
   dwmdrm = -wm^2 * Li; 
   
   % Find the scaled radius rsm = rs(rm) satisfying ws(rsm) = wm, by bisection
   rsm = willoughby1_rsm(a);
   
 % dwsdrs_rsm = 0.9375*(rsm^2 - 1)^2;  removed 19.1.2005 seemingly unused
 % dwsdrsi_rsm = 1/dwsdrs_rsm;
   
   rblend1 = rm - Lb*rsm - Lb;
   rblend2 = rm - Lb*rsm + Lb;
end

if rblend1 < 0
   disp(sprintf('WARNING willoughby1_p_numerical >> rblend1 = %f < 0', rblend1));
   p = repmat(nan, size(r));
   return;
end

rho = 1.15;
rs = ( r - rm )/Lb + rsm;   % Scaled r
rrm = r/rm;

rmax = max(r);
nr = length(r);
p  = zeros(nr,1) + nan;
dp = zeros(nr,4) + nan;

% Inner zone - from r=0 to start of blending
ix = find( 0 <= r & r <= rblend1 );
p(ix) = pc + rho*0.5*vm*(vm + f*rm)*rrm(ix).^2;
dp(ix,1) = rho*(vm + 0.5*f*rm)*rrm(ix).^2;
dp(ix,2) = 0;
dp(ix,3) = -rho*vm*(vm/rm + 0.5*f)*rrm(ix).^2;
dp(ix,4) = 1;

pblend1 = pc + rho*0.5*vm*(vm + f*rm)*(rblend1/rm)^2;

% Blending zone
ix = find( rblend1 < r & r <= rblend2 );
ndum = 50;
rdum = ([ 0 : ndum ]')*2*Lb/ndum + rblend1;
[vdum, dvdum] = willoughby1_v(rdum, a(1:3));
gwe = vdum .* ( vdum./rdum + f );
pdum = pblend1 + rho*cumint(gwe, rdum(2)-rdum(1));
if ~isempty(ix)
   p(ix) = interp1(rdum,pdum,r(ix),'spline');
end
if 0 & (nargout >= 2)  % Calculate derivatives
   Iterm = 2*vdum./rdum + f;
   for ii = 1 : 3
      dpdum = rho*cumint( Iterm .* dvdum(:,ii), dr );
      dp(ix,ii) = interp1(rdum,dpdum,r(ix),'spline');
   end
   dp(ix,4) = 1;
end

pblend2 = pdum(ndum+1);

% Outer zone
ix = find( rblend2 < r );
Ei1  = -real(expint(2*r(ix)*Li)) + real(expint(2*rblend2*Li));
e1m = exp(rm*Li); e1 = exp((rm - r(ix))*Li); e1b = exp((rm - rblend2)*Li);
p(ix) = pblend2 + rho*( f*vm*L*(e1b - e1) + (vm*e1m)^2 * Ei1  );
if 0
   dp(ix,1) = rho*( (vm + 0.5*f*rm) + f*L*(1 - e1) + 2*vm*e1m^2*Ei1 );
   dp(ix,2) = rho*vm*( f*( 1 - e1.*( 1 - Li*(rm - r(ix)) ) ) ...
      + vm*Li*( (1 - e1.^2) - 2*rm*Li*e1m^2*Ei1 ) );
   dp(ix,3) = nan; % needs fixing
   dp(ix,4) = 1;
end

% Special points
ix = find(r == -999.0);
rmhat = 25e3; rmscal = 1;
Lhat = 7e4;  Lscal = 1;
p(ix) = rmscal*(rm/rmhat - 1)^4 + Lscal*(L/Lhat - 1)^4;
dp(ix,1) = 0;
dp(ix,2) = 4*Lscal/Lhat*(L/Lhat - 1)^3;
dp(ix,3) = 4*rmscal/rmhat*(rm/rmhat - 1)^3;
dp(ix,4) = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function dpdr = gwe(r,p,a,f)

rho = 1.15;
v =  willoughby1_v(r, a(1:3));
dpdr = rho .* v .* ( v./(r + 1e-100) + f );




