function [p,dp] = willoughby2_p_noblend(r,a,f);
% function [p,dp] = willoughby2_p_noblend(r,a,f);
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm) and its derivatives
% with respect to the control vector together with the radial integral of the
% gradient wind equation i.e. the corresponding pressure field.
% *** Simplified version omits the blending zone between v1 and v2. ***
% The control vector a = (A,L1,B,L2,rm,n1,pc,T,Lb)

if (nargin < 3)
   global f0;
   f = f0;
end
A = a(1);    % amplitude of first exponential
L1 = a(2);   % length scale of first exponential
B = a(3);    % amplitude of second exponential
L2 = a(4);   % length scale of second exponential
rm = a(5);   % radius of maximum winds
n1 = a(6);   % exponent for centre profile (v1)
pc = a(7);   % central pressure
T  = a(8);   % mean virtual temperature
if length(a) > 8
   Lb = a(9);   % Half-width of blending zone
else
   Lb = 5e3;
end
R  = 287;
RT = R*T;
RTi = 1/RT;
L1i = 1/L1; L2i = 1/L2;
L12i = L1i + L2i;

nr = length(r);

rrm = r/rm;
p  = zeros(nr,1);
if nargout >= 2
   dp = zeros(nr,9);
end

% Centre
ix = find( r == 0 );
if ~isempty(ix)
   p(ix) = pc;
   if nargout >= 2
      dp(ix,1) = 0;
      dp(ix,2) = 0;
      dp(ix,3) = 0;
      dp(ix,4) = 0;
      dp(ix,5) = 0;
      dp(ix,6) = 0;
      dp(ix,7) = 1;
      dp(ix,8) = 0;
      dp(ix,9) = 0;
   end
end

% Inner zone
ix = find( 0 < r & r <= rm );
if ~isempty(ix)
   rrmn = rrm(ix).^n1;
   gwe1 = ((A + B)/(2*n1*(n1 + 1))) ...
      * (2*f*n1*r(ix) + (A + B)*(1 + n1)*rrmn).*rrmn;
   ee1 = exp(RTi*gwe1);
   p(ix) = pc*ee1;
   if nargout >= 2
      dp(ix,1) = (RTi/(n1*(n1 + 1))) * (f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*rrmn.*p(ix);
      dp(ix,2) = 0;
      dp(ix,3) = dp(ix,1);
      dp(ix,4) = 0;
      dp(ix,5) = (-RTi*(A + B)/((n1 + 1)*rm)) * (f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*rrmn.*p(ix);
      dp(ix,6) = ((A + B)*RTi*rrmn/(2*(n1 + n1^2)^2)) .* ...
         (-2*f*n1^2*r(ix) - (A + B)*(n1 + 1)^2*rrmn ...
           + 2*n1*(n1 + 1)*(f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*log(rrm(ix))).*p(ix);
      dp(ix,7) = ee1;
      dp(ix,8) = -gwe1 .* p(ix) / (R*T^2);
      dp(ix,9) = (A + B)*RTi*(f + (A + B)*rrmn./r(ix)).*rrmn.*p(ix);
   end
end

% Outer zone
ix = find( rm < r );
if ~isempty(ix)
   Ei1  = -real(expint(2*r(ix)*L1i)) + real(expint(2*rm*L1i));
   Ei2  = -real(expint(2*r(ix)*L2i)) + real(expint(2*rm*L2i));
   Ei12 = -real(expint(r(ix)*L12i)) + real(expint(rm*L12i));
   e1m = exp(rm*L1i); e1 = exp((rm - r(ix))*L1i);
   e2m = exp(rm*L2i); e2 = exp((rm - r(ix))*L2i);
   e12m = exp(rm*L12i); e12 = exp((rm - r(ix))*L12i);
   gwe2 = ((A + B)*((A + B)*(n1 + 1) + 2*f*n1*rm)/(2*n1*(n1 + 1))) ...
      + f*( A*L1*(1 - e1) + B*L2*(1 - e2) ) ...
      + (A*e1m)^2 * Ei1 + (B*e2m)^2 * Ei2 + 2*A*B * e12m * Ei12;
   ee2 = exp(RTi*gwe2);
   p(ix) = pc * ee2;
   if nargout >= 2
      dp(ix,1) = RTi*( (A + B)/n1 + f*rm/(1 + n1) ...
         + f*L1*(1 - e1) + 2*A*e1m^2*Ei1 + 2*B*e12m*Ei12 ).*p(ix);
      dp(ix,3) = RTi*( (A + B)/n1 + f*rm/(1 + n1) ...
         + f*L2*(1 - e2) + 2*B*e2m^2*Ei2 + 2*A*e12m*Ei12 ).*p(ix);
      dp(ix,2) = RTi*A*( f*( 1 - e1.*( 1 - L1i*(rm - r(ix)) ) ) ...
         + A*L1i*( (1 - e1.^2) - 2*rm*L1i*e1m^2*Ei1 ) ...
         + 2*B*L1i*( L2*(1 - e12)/(L1+L2) - rm*e12m*L1i*Ei12 ) ).*p(ix);
      dp(ix,4) = RTi*B*( f*( 1 - e2.*( 1 - L2i*(rm - r(ix)) ) ) ...
         + B*L2i*( (1 - e2.^2) - 2*rm*L2i*e2m^2*Ei2 ) ...
         + 2*A*L2i*( L1*(1 - e12)/(L1+L2) - rm*e12m*L2i*Ei12 ) ).*p(ix);
      dp(ix,5) = RTi*( f*(A + B)/(n1 + 1) - f*(A*e1 + B*e2) - ((A + B)^2)/rm ...
         + 2*(A*e1m)^2*L1i*Ei1 + 2*(B*e2m)^2*L2i*Ei2 ...
         + 2*A*B*e12m*L12i*Ei12 ).*p(ix);
      dp(ix,6) = -RTi*(A + B)*((A + B)/(2*n1^2) + f*rm/(n1 + 1)^2)*p(ix);
      dp(ix,7) = ee2;
      dp(ix,8) = -(1 / (R*T^2)) * p(ix) .* gwe2;
      dp(ix,9) = RTi*p(ix).*(A*e1 + B*e2).*((A*e1 + B*e2)./r(ix) + f);
   end
end

% Special points
ix = find(r == -999.0);
if ~isempty(ix)
   global w2p_rmhat w2p_rmscal w2p_L1hat w2p_L1scal w2p_L2hat w2p_L2scal
   w2p_rmhat = 25e3; w2p_rmscal = 1;
   w2p_L1hat = 7e4;  w2p_L1scal = 1;
   w2p_L2hat = 4e5;  w2p_L2scal = 1;
   p(ix) = w2p_rmscal*(rm/w2p_rmhat - 1)^4 ...
      + w2p_L1scal*(L1/w2p_L1hat - 1)^4 ...
      + w2p_L2scal*(L2/w2p_L2hat - 1)^4;
   if nargout >= 2
      dp(ix,1) = 0;
      dp(ix,2) = 4*w2p_L1scal/w2p_L1hat*(L1/w2p_L1hat - 1)^3;
      dp(ix,3) = 0;
      dp(ix,4) = 4*w2p_L2scal/w2p_L2hat*(L2/w2p_L2hat - 1)^3;
      dp(ix,5) = 4*w2p_rmscal/w2p_rmhat*(rm/w2p_rmhat - 1)^3;
      dp(ix,6) = 0;
      dp(ix,7) = 0;
      dp(ix,8) = 0;
      dp(ix,9) = nan;
   end
end


