function [v,dv,wzone] = willoughby2_v(r,a)
% function [v,dv,wzone] = willoughby2_v(r,a)
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm) and its derivatives
% with respect to the control vector. 
% The centre of the blending zone is set through the global variable w2_p_rblend
% rather than calculated from the input parameters.
% The control vector a = (A,L1,B,L2,rm,n1,pc,T,Lb)

global w2_fix_rblend

A  = a(1);    % amplitude of first exponential
L1 = a(2);   % length scale of first exponential
B  = a(3);    % amplitude of second exponential
L2 = a(4);   % length scale of second exponential
rm = a(5);   % radius of maximum winds
n1 = a(6);   % Exponent for v1 (inner zone)
pc = a(7);   % central pressure (not used)
RT = 287*a(8);  % for density terms (not used)
if length(a) > 8
   Lb = a(9);   % Half-width of blending zone
else
   Lb = 5e3;
end
 
rs = ( r - w2_fix_rblend )/Lb;   % Scaled r for blending

nr  = length(r); na = 8;

w  = zeros(size(r)); 
ix = find(abs(rs) < 1);
%% Old version has 2 continuous derivatives, new one has 4. 
%% w(ix) = 0.5 + 0.0625*rs(ix).*(3*rs(ix).^4 - 10*rs(ix).^2 + 15);
w(ix) = (1/256) * (1 + rs(ix)).^5 .* (128 - rs(ix).*(325 - rs(ix).*(345 - rs(ix).*(175 - rs(ix)*35))));
w( find(rs >= 1) ) = 1;

rrm = r/rm;
e1 = exp((rm - r)/L1); 
e2 = exp((rm - r)/L2);

ix = find(r ~= -999.0);  % Normal points

rrmn = rrm.^n1;
v1 = (A + B)*rrmn;
v2 = A*e1 + B*e2;
v = (1 - w).*v1 + w.*v2;

if nargout >= 2
   dv1 = zeros(nr,na);
   dv1(:,1) = rrmn;
   dv1(:,2) = 0;
   dv1(:,3) = rrmn;
   dv1(:,4) = 0;
   dv1(:,5) = -(n1/rm)*v1;
   ix = find(r >  0); dv1(ix,6) = log(rrm(ix)).*v1(ix);
   ix = find(r == 0); dv1(ix,6) = 0;
   dv1(:,7) = 0;
   dv1(:,8) = 0;
   
   dv2 = zeros(nr,na);
   dv2(:,1) = e1;
   dv2(:,2) = -A*(rm - r).*e1/L1^2;
   dv2(:,3) = e2;
   dv2(:,4) = -B*(rm - r).*e2/L2^2;
   dv2(:,5) = A*e1/L1 + B*e2/L2;
   dv2(:,6) = 0;
   dv2(:,7) = 0;
   dv2(:,8) = 0;

   oa = ones(1,na);
   dv = (1 - w*oa).*dv1 + w*oa.*dv2;
end

ix = find(r == -999.0);
if ~isempty(ix) 
   % Special point, put in penalty term on length scales etc.
   global w2v_rmhat w2v_rmscal w2v_L1hat w2v_L1scal w2v_L2hat w2v_L2scal
   v(ix) = w2v_rmscal*(rm/w2v_rmhat - 1)^2 ...
      + w2v_L1scal*(L1/w2v_L1hat - 1)^4 ...
      + w2v_L2scal*(L2/w2v_L2hat - 1)^4;
   if nargout >= 2
      dv(ix,1) = 0;
      dv(ix,2) = 4*w2v_L1scal/w2v_L1hat*(L1/w2v_L1hat - 1)^3;
      dv(ix,3) = 0;
      dv(ix,4) = 4*w2v_L2scal/w2v_L2hat*(L2/w2v_L2hat - 1)^3;
      dv(ix,5) =   w2v_rmscal/w2v_rmhat*(rm/w2v_rmhat - 1);
      dv(ix,6) = 0;
      dv(ix,7) = 0;
      dv(ix,8) = 0;
   end
end

