function [v,dv] = willoughby2s_v(r,a)
% function [v,dv] = willoughby2s_v(r,a)
% Gives the analytical wind profile of Willoughby (2002) in the
% two-exponential form, with 5-th order weighting function, for
% control vector a = [A L1 B L2 rm]', together with the
% derivatives with respect to the control vector dv/da.
% Simplified version doesn't calculate location of weighting function.

A = a(1);
L1 = a(2);
B = a(3);
L2 = a(4);
rm = a(5);

nr = length(r); na = length(a);

v1 = zeros(nr,1); dv1 = zeros(nr,na);
v2 = zeros(nr,1); dv2 = zeros(nr,na);
w  = zeros(nr,1); dwdrm = zeros(nr,1);
oa = ones(1,na);

for ir = 1 : nr
   if (r(ir) ~= -999.0)  % Normal points
      rrm = r(ir)/rm;
      e1 = exp((rm - r(ir))/L1); 
      e2 = exp((rm - r(ir))/L2);

      % rs = 2*(rrm - 0.75);
      % drsdrm = -2*rrm/rm;
      rs = 4*(rrm - 0.875);
      drsdrm = -4*rrm/rm;

      v1(ir) = (A + B)*rrm;
      dv1(ir,1) = rrm;
      dv1(ir,2) = 0;
      dv1(ir,3) = rrm;
      dv1(ir,4) = 0;
      dv1(ir,5) = -(A + B)*rrm/rm;

      v2(ir) = A*e1 + B*e2;
      dv2(ir,1) = e1;
      dv2(ir,2) = -A*(rm - r(ir))*e1/L1^2;
      dv2(ir,3) = e2;
      dv2(ir,4) = -B*(rm - r(ir))*e2/L2^2;
      dv2(ir,5) = A*e1/L1 + B*e2/L2;

      if (rs < -1)    
         w(ir) = 0;
         dwdrm(ir) = 0;
      elseif (rs < 1) 
       % w(ir) = 0.5 - 0.25*rs*(rs^2 - 3);
       % dwdrm(ir) = -0.75*(rs^2 - 1)*drsdrm;
         w(ir) = 0.5 + 0.0625*rs.*(3*rs^4 - 10*rs^2 + 15);
         dwdrm(ir) = 0.9375*(rs^2 - 1)^2 * drsdrm;
      else            
         w(ir) = 1;
         dwdrm(ir) = 0;
      end;
   else % Special point, put in penalty term on length scales etc.
      rmhat = 25e3;
    % v1(ir) = 1e-4*(L1/40 - 1)^4 + 1e-2*(L2/300 - 1)^4 + 1e-4*(rm/rmhat - 1)^4;
      v1(ir) = 1e-2*(rm/rmhat - 1)^4;
      dv1(ir,1) = 0;
      dv1(ir,2) = 0; % 4e-4/40*(L1/40 - 1)^3;
      dv1(ir,3) = 0;
      dv1(ir,4) = 0; % 4e-2/300*(L2/300 - 1)^3;
      dv1(ir,5) = 4e-3/rmhat*(rm/rmhat - 1)^3;
      v2(ir) = 0;
      dv2(ir,:) = 0;
      dwdrm(ir) = 0;
      w(ir) = 0;
   end
end

v = (1 - w).*v1 + w.*v2;
dv = (1 - w*oa).*dv1 + w*oa.*dv2;
dv(:,5) = dv(:,5) + dwdrm .* (v2 - v1);

