function [anal, err] = xsecsi(rg,zg,Tb,ro,zo,To,Lr,Lz,sigb,sigo)
% Does a 2-dimensional univariate statistical interpolation analysis of 
% observations To at locations (ro,zo) onto a grid (rg,zg) with background
% field Tb. (rg,zg) need to have been made using meshgrid and have constant
% spacing.
% Background error covariances are modelled in cartesian coordinates with 
% length scales Lr and Lz, and variance sigb^2. Observation errors
% are assumed uncorrelated with variance sigo^2.
% Follows somewhere between Lorenc (1981) and  Talagrand (1997) section 4.

calcB = 0;   % Calculate off-diagonal terms of B

sigb2 = sigb^2; sigo2 = sigo^2;

[nz,nr] = size(Tb); nrz = nr*nz;
dr = rg(1,2) - rg(1,1); dz = zg(2,1) - zg(1,1);
rmin = min(rg(:)); rmax = max(rg(:));
zmin = min(zg(:)); zmax = max(zg(:));

no = length(ro);

H = zeros(no,nrz);
Hx = reshape( 1:nrz , [nz,nr] );
for io = 1 : no
   if (ro(io) >= rmin & ro(io) <= rmax & zo(io) >= zmin & zo(io) <= zmax);
      ir = max( find( rg(1,:) <= ro(io) ) );
      iz = max( find( zg(:,1) <= zo(io) ) );
      r1 = ( -rg(iz,ir) + ro(io) ) / dr; r2 = 1 - r1;
      z1 = ( -zg(iz,ir) + zo(io) ) / dz; z2 = 1 - z1;
      H( io, Hx(iz  ,ir  ) ) = r2*z2;
      H( io, Hx(iz+1,ir  ) ) = r2*z1;
      H( io, Hx(iz  ,ir+1) ) = r1*z2;
      H( io, Hx(iz+1,ir+1) ) = r1*z1;
   end
end;

C = zeros(no);
for io = 1 : no
   C(io,io) = sigb2 + sigo2;
   for jo = io+1 : no
      C(io,jo) = sigb2 * exp( -0.5*dsq( ro(io),zo(io),ro(jo),zo(jo),Lr,Lz ) );
      C(jo,io) = C(io,jo);
   end
end
Ci = inv(C);

% Background error matrix (only used for plotting)
B = sigb2*eye(nrz);    
if calcB
   for ig = 1 : nrz
      for jg = ig+1 : nrz
         B(ig,jg) = sigb2 * exp( -0.5*dsq( rg(ig),zg(ig),rg(jg),zg(jg),Lr,Lz ) );
         B(jg,ig) = B(ig,jg);
      end
   end
end

HB = zeros(no,nrz);
for io = 1 : no
   for kg = 1 : nrz
      HB(io,kg) = sigb2 * exp( -0.5*dsq( ro(io),zo(io),rg(kg),zg(kg),Lr,Lz ) );
   end
end

Tainc = HB' * Ci * ( To - H*(Tb(:)) );  % Analysis increments
anal = Tb + reshape( Tainc, [nz,nr] );  % Analysis

Pa = B - HB' * Ci * HB;                 % Analysis error covariance matrix
err = reshape( diag(Pa), [nz,nr] );     % Analysis error variances

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function dd = dsq(r1,z1,r2,z2,Lr,Lz)
% Returns the square of the nondimensional polar distance between two points

dd = ( (r1 - r2)/Lr )^2 + ( (z1 - z2) / Lz )^2;   % Lz in radians

